﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Text;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PuzzlePlayer_Namespace
{
    internal class PuzzleForm : Form
    {
        private Button solvebutton;
        private Button hintbutton;
        private Button generatebutton;
        private TextBox informationbox;
        private Graphics graphics;
        private Rectangle boardspace;

        private Board board;
        public Board Board //Updating the Board member will immediately call board.Draw method so that the board is updated visually
        {
            get { return board; }
            set
            {
                board = value;
                this.Invalidate();
            }
        }

        public PuzzleForm(Board b = null, Size s = default(Size)) //Takes Board and Size parameter and sets up the PuzzleForm window.
        {
            if (s == default(Size)) s = new Size(700, 420);
            this.Size = this.MinimumSize = s;
            this.WindowState = FormWindowState.Maximized;
            this.BackColor = Color.DarkGreen;
            this.Text = "top text";
            graphics = this.CreateGraphics();
            graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            generatebutton = new Button();
            hintbutton = new Button();
            solvebutton = new Button();
            informationbox = new TextBox();
            boardspace = new Rectangle(220, 30, 400, 400);
            this.Paint += (object o, PaintEventArgs pea) =>
            {
                board.Draw(graphics, boardspace);
                //graphics.FillRectangle(Brushes.LightCoral, 220, 30, this.Width - 380, this.Height - 100);
                //graphics.FillRectangle(Brushes.DarkRed, boardspace);
            };
            this.Resize += (object o, EventArgs ea) => UpdateUI();
            Board = b;
            CreateUI();
            Board.boardState[1, 1] = 1;
            Board.boardState[2, 2] = 0;
        }
        private void CreateUI() //sets up ui elements
        {
            void CreateButton(Button b)
            {
                Controls.Add(b);
                b.Size = new Size(80, 50);
                b.BackColor = Color.Gainsboro;
                b.Text = "DEFAULT0";
            }
            CreateButton(generatebutton);
            generatebutton.Text = "Generate";
            generatebutton.MouseClick += (object sender, MouseEventArgs mea) => 
            {
                //board = Board.Generate();
            };
            CreateButton(hintbutton);
            hintbutton.Text = "Hint";
            hintbutton.MouseClick += (object sender, MouseEventArgs mea) =>
            {
                MessageBox.Show("Hint: geef op");
            };
            CreateButton(solvebutton);
            solvebutton.Text = "Solve";
            solvebutton.MouseClick += (object sender, MouseEventArgs mea) =>
            {
                //board = board.Solve();
                Board = Board;
            };

            Controls.Add(informationbox);
            informationbox.Multiline = true;
            informationbox.Size = new Size(160, 300);
            informationbox.BackColor = Color.LightGray;
            informationbox.Text = board.description;
            informationbox.Font = new Font("Verdana", 10);
            UpdateUI();
        }
        private void UpdateUI() //resizes the boardspace rectangle and updates the rest of the ui around the new boardspace size
        {
            graphics.Clear(this.BackColor);
            boardspace.Size = FitBoard(new Size(Board.boardState.GetLength(0),Board.boardState.GetLength(1)),new Size(this.Width - 380, this.Height - 100),8);
            generatebutton.Location = new Point(boardspace.Right + 30, 30);
            hintbutton.Location = new Point(boardspace.Right + 30, 130);
            solvebutton.Location = new Point(boardspace.Right + 30, 230);
            informationbox.Location = new Point(40, 30);
            this.Invalidate();
        }
        public static Size FitBoard(Size gamesize, Size maxboardsize, int drawfactor) //returns the largest rectangle smaller than MaxBoardSize that fits the given GameSize well
        {
            int TileLength = Math.Min(maxboardsize.Width / gamesize.Width, maxboardsize.Height / gamesize.Height) / drawfactor * drawfactor;
            return new Size(gamesize.Width*TileLength, gamesize.Height*TileLength);
            //After each of the divisions in this method, the value should be rounded down,
            //as the end value cannot exceed the size given by maxboardsize.
            //however, C# rounds divisions of Int values down by default, so this does not have to be explicitly done.
        }
        public static Point GetTile(Point p)
        {
            return p;
        }
    }
}
